searchData={"items":[{"type":"module","title":"Jaqex","doc":"Documentation for `Jaqex`.","ref":"Jaqex.html"},{"type":"function","title":"Jaqex.filter/3","doc":"Filters the given JSON string with the given j(a)q code and loading `.jq` scripts\nin the given path.","ref":"Jaqex.html#filter/3"},{"type":"function","title":"Examples - Jaqex.filter/3","doc":"```\niex> Jaqex.filter(\"[1, 2, 3]\", \"[ .[] | {v: .} ]\")\n{:ok, [%{\"v\" => 1}, %{\"v\" => 2}, %{\"v\" => 3}]}\n```\n\nAssuming `priv/` contains a jq script `t.jq`, the following demonstrates Jaqex loading that\nand availing it for use by your filter code:\n\n```\niex> Jaqex.filter(\"[\\\"fooBar\\\"]\", \"import \\\"t\\\" as t; [ .[] | t::snake_case(.) ]\", \"priv\")\n{:ok, [\"foo_bar\"]}\n```","ref":"Jaqex.html#filter/3-examples"},{"type":"function","title":"Jaqex.filter!/3","doc":"Similar to `filter/3` but raises on errors.","ref":"Jaqex.html#filter!/3"},{"type":"function","title":"Jaqex.filter_file/3","doc":"Similar to `filter/3` but loads the json doc at the given path. Note that the\nfile is opened in \"Rust-land\", thus bypassing any potential issues the BEAM may\ncause when processing large binaries.","ref":"Jaqex.html#filter_file/3"},{"type":"function","title":"Jaqex.filter_file!/3","doc":"Similar to `filter_file/3` but raises on errors.","ref":"Jaqex.html#filter_file!/3"},{"type":"extras","title":"Jaqex","doc":"# Jaqex\nElixir wrapper for [jaq](https://github.com/01mf02/jaq), a [jq](https://jqlang.github.io/jq/)\nclone focused on correctness, speed, and simplicity implemented in Rust.\n\nUnlike other jq wrappers, this doesn't require jq to be installed/available in `$PATH`:\nJaqex uses [rustler](https://hexdocs.pm/rustler/) to implement the NIFs to jaq.\nAnd because Jaqex uses jaq, certain jq features like SQL-style operators aren't available.\nPlease refer to [jaq's README \"differences between jq and jaq\"][diff]\nfor more information. Despite such differences, jaq should still be useful in most\nscenarios, especially in the succintness in filtering/transforming JSON.\n\nThe following demonstrates the difference between using Elixir/Jason and Jaqex. Assuming\nwe have transform the following JSON doc from this:\n\n```json\n{\n    \"ticker\": \"AAPL\",\n    \"adjusted\": true,\n    \"data\": [\n      [229.52, 229.65, 223.74, 226.21, \"2024-10-01\"],\n      [225.89, 227.37, 223.02, 226.78, \"2024-10-02\"]\n    ],\n    \"fields\": [\n      {\"field\": \"open\", \"type\": \"float\"},\n      {\"field\": \"high\", \"type\": \"float\"},\n      {\"field\": \"low\",  \"type\": \"float\"},\n      {\"field\": \"close\",\"type\": \"float\"},\n      {\"field\": \"date\", \"type\": \"date\"}\n    ]\n}\n```\n\nTo this:\n\n```elixir\n[\n    %{\n        \"open\" => 229.52, \"high\": 229.65, \"low\" => 223.74, \"close\" => 226.21,\n        \"date\" => \"2024-10-01\", \"ticker\" => \"AAPL\", \"adjusted\": true\n    },\n    %{\n        \"open\" => 225.89, \"high\": 227.37, \"low\" => 223.02, \"close\" => 226.78,\n        \"date\" => \"2024-10-02\", \"ticker\" => \"AAPL\", \"adjusted\": true\n   },\n]\n```\n\nAssuming the positions of the values aren't fixed and there may be additions to\nthe fields and values, the following shows how it's done using Elixir and Jason:\n\n```elixir\ndoc = Jason.decode!(json_string)    # assuming the contents are in json_string\nkeys = doc[\"fields\"] |> Enum.map(&Map.get(&1, \"field\"))\nresult =\n    doc[\"data\"]\n    |> Stream.map(&Enum.zip(keys, &1))\n    |> Stream.map(&Map.new/1)\n    |> Enum.map(&Map.merge(&1, additions))\n```\n\nComparing that with using Jaqex:\n\n```elixir\nresult = Jaqex.filter!(\n    json_string,\n    \"[. as $root\n      | .data[]\n      | [[$root.fields[] | .field], .]\n      | transpose\n      | map({key: .[0], value: .[1]})\n      | from_entries\n      | . + {ticker: $root.ticker, adjusted: $root.adjusted}\n    ]\"\n)\n```\n\nJaq/jq filters make transformations as such straightforward (when you know jq-lang).\nBecause Jaq's transformations are done in Rust, memory usage is much lower and transformation\nfaster; native Elixir version requires creating multiple (interim/throwaway) lists and\nmaps (`keys`, `additions`, the maps from `&Enum.zip(keys, &1)` and `&Map.new/1`).","ref":"readme.html"},{"type":"extras","title":"Installation - Jaqex","doc":"Add `jaqex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:jaqex, \"~> 0.1.3\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Copyright and License - Jaqex","doc":"Copyright © 2024, Shaolang Ai\n\nDistributed under the MIT License\n\n[diff]: https://github.com/01mf02/jaq/blob/main/README.md#differences-between-jq-and-jaq","ref":"readme.html#copyright-and-license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}